@@ -1,10 +1,8 @@
-from pathlib import Path
-
 import click
 
 from indico_install.cluster_manager import ClusterManager
+from indico_install.config import D_PATH
 from indico_install.kube.svc.restart import restart
-from indico_install.setup import configure_gcr_key
 from indico_install.utils import options_wrapper, run_cmd
 
 
@@ -59,10 +57,8 @@ def apply(
     ctx,
     restart_deps=True,
     services=None,
-    deployment_root=None,
     yes=False,
     cluster_manager=None,
-    **kwargs,
 ):
     """
     Apply templates in the "generated/" directory.
@@ -70,13 +66,11 @@ def apply(
     Creates the image pull secret if missing.
     Performs a dry-run for confirmation before applying. Skip dry-run with --yes
     """
-    generated_dir = Path(deployment_root) / "generated"
+    generated_dir = D_PATH / "generated"
     if not generated_dir.is_dir():
-        click.secho(f"'generated/' directory not found in {deployment_root}")
+        click.secho(f"'generated/' directory not found in {D_PATH}")
         return
 
-    configure_gcr_key(deployment_root)
-
     # These are services that were actually modified
     updated_services = []
 
