@@ -1,16 +1,10 @@
 import click
-import untangle
-import base64
 import json
-import re
-import sys
-from subprocess import Popen, PIPE, STDOUT
-from OpenSSL import crypto, SSL
-from urllib.error import HTTPError
+from OpenSSL import crypto
 from onelogin.saml2.idp_metadata_parser import OneLogin_Saml2_IdPMetadataParser
+from click import prompt, secho
 
-
-from indico_install.utils import options_wrapper, run_cmd, convertb64
+from indico_install.utils import run_cmd
 from indico_install.infra.input_utils import download_indicoapi_data, auth_with_gsutil
 from indico_install.kube.svc.restart import restart
 from indico_install.infra.single_node import single_node
@@ -41,6 +35,24 @@ def infra(ctx):
     pass
 
 
+def update_secret(secret_name, namespace="default"):
+    """
+    Update an existing secret
+    """
+    if run_cmd(f"kubectl get secrets -n {namespace} | grep {secret_name}", silent=True):
+        del_secret = prompt(
+            f"Generate secret: '{secret_name}'? If 'yes' existing secret will be deleted",
+            type=bool,
+            default=False,
+        )
+        if del_secret:
+            run_cmd(f"kubectl delete secret -n {namespace} {secret_name}")
+            return True
+        else:
+            return False
+    else:
+        return True
+
 def generate_cert(
     country="US",
     state="MA",
@@ -73,7 +85,25 @@ def generate_cert(
     }
 
 
-@infra.command("generate-saml")
+@infra.command("generate-saml-cert")
+@click.pass_context
+@click.argument("host", required=True, type=str)
+def generate_saml_cert(ctx, host=None):
+    """
+    Generate key/certificate for SSO
+    """
+    if not update_secret("indico-sso-cert"):
+        return
+    crypto_output = generate_cert(common_name=host)
+    secho("Generating key/certificate", fg="green")
+    run_cmd(
+        f"""kubectl create secret generic indico-sso-cert --from-literal=sp.key="{crypto_output.get('key')}" --from-literal=sp.crt="{crypto_output.get('crt')}" """,
+        silent=True,
+    )
+    ctx.invoke(restart, services=["noct"], contains=False, wait="2m")
+
+
+@infra.command("generate-saml-settings")
 @click.pass_context
 @click.argument("host", required=True, type=str)
 @click.argument("metadata_url", required=False, type=str)
@@ -82,7 +112,14 @@ def generate_cert(
     "--custom-saml",
     help="Optional path to a saml settings.json file; if specified, will upload the given file as a noct secret",
 )
-def generate_saml(ctx, host=None, metadata_url=None, custom_saml=None):
+@click.option(
+    "-a",
+    "--advanced-settings",
+    help="Optional path to a saml advanced_settings.json file; if specified, will upload the given file as a noct secret",
+)
+def generate_saml(
+    ctx, host=None, metadata_url=None, custom_saml=None, advanced_settings=None
+):
     """
     Generate and inject SAML settings from Azure AD Saml2
     Before running, complete the following steps:
@@ -98,23 +135,43 @@ def generate_saml(ctx, host=None, metadata_url=None, custom_saml=None):
     Alternatively, inject a custom settings.json file:
     indico infra generate-saml -f custom-settings.json
 
+    An 'advanced_settings.json' file can be added with:
+    indico infra generate-saml -f custom-settings.json -a advanced_settings.json
+
+    See here for more details: https://github.com/onelogin/python3-saml
+
     """
-    for secret in ["indico-sso-secrets", "indico-sso-cert"]:
-        run_cmd(f"kubectl delete secret {secret}")
-
-    crypto_output = generate_cert(common_name=host)
-
-    run_cmd(
-        f'''kubectl create secret generic indico-sso-cert --from-literal=sp.key="{crypto_output.get('key')}" --from-literal=sp.crt="{crypto_output.get('crt')}" ''', silent=True
+    settings_secret_name = "indico-sso-settings"
+    if run_cmd(f"kubectl get secrets | grep {settings_secret_name}", silent=True):
+        del_secret = prompt(
+            f"Secret named '{settings_secret_name}' already exists; remove and create a new secret?",
+            type=bool,
+            default=False,
         )
-
+        if del_secret:
+            run_cmd(f"kubectl delete secret {settings_secret_name}")
+        else:
+            return
+    # load advanced_settings json if it exists
+    adv_settings_content = ""
+    if advanced_settings:
+        with open(advanced_settings) as settings_file:
+            adv_settings = json.load(settings_file)
+        adv_settings_content = json.dumps(adv_settings, indent=4)
+    advanced_secret = (
+        f"--from-literal=advanced_settings.json='{adv_settings_content}'"
+        if adv_settings_content
+        else ""
+    )
     if custom_saml:
         with open(custom_saml) as settings_file:
             settings = json.load(settings_file)
         settings_content = json.dumps(settings, indent=4)
         run_cmd(
-            f'''kubectl create secret generic indico-sso-secrets --from-literal=settings.json='{settings_content}' ''', silent=True
-            )
+            f"""kubectl create secret generic {settings_secret_name} --from-literal=settings.json='{settings_content}' """
+            + f"{advanced_secret}",
+            silent=True,
+        )
         ctx.invoke(restart, services=["noct"], contains=False, wait="2m")
         return
 
@@ -126,14 +183,27 @@ def generate_saml(ctx, host=None, metadata_url=None, custom_saml=None):
         )
         return
     idp_info = OneLogin_Saml2_IdPMetadataParser.parse_remote(metadata_url)
-    jacs = { "url": "https://" + host + "/auth/users/saml/acs", "binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST" }
-    jsls = { "url": "https://" + host + "/auth/users/saml/sls", "binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect" }
-    jsp = { "entityId": "https://" + host + "/auth/users/saml/metadata", "assertionConsumerService": jacs, "singleLogoutService": jsls, "NameIDFormat": "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress" }
-    stg = { "strict": "true", "debug": "true", "sp": jsp, "idp": idp_info["idp"] }
+    jacs = {
+        "url": "https://" + host + "/auth/users/saml/acs",
+        "binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST",
+    }
+    jsls = {
+        "url": "https://" + host + "/auth/users/saml/sls",
+        "binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect",
+    }
+    jsp = {
+        "entityId": "https://" + host + "/auth/users/saml/metadata",
+        "assertionConsumerService": jacs,
+        "singleLogoutService": jsls,
+        "NameIDFormat": "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress",
+    }
+    stg = {"strict": "true", "debug": "true", "sp": jsp, "idp": idp_info["idp"]}
     settings_content = json.dumps(stg, indent=4)
     run_cmd(
-        f'''kubectl create secret generic indico-sso-secrets --from-literal=settings.json='{settings_content}' ''', silent=True
-        )
+        f"""kubectl create secret generic {settings_secret_name} --from-literal=settings.json='{settings_content}' """
+        + f"{advanced_secret}",
+        silent=True,
+    )
     ctx.invoke(restart, services=["noct"], contains=False, wait="2m")
 
 
@@ -146,8 +216,7 @@ def generate_saml(ctx, host=None, metadata_url=None, custom_saml=None):
     show_default=True,
     help="Automatically extract the downloaded TAR",
 )
-@options_wrapper()
-def download_api_data(ctx, version, *, extract, deployment_root, **kwargs):
+def download_api_data(ctx, version, *, extract):
     """
     Download VERSION of API data TAR from google cloud to local --deployment-root.
     Un-tar the file into a directory of the same name, also in the deployment-root
@@ -158,9 +227,26 @@ def download_api_data(ctx, version, *, extract, deployment_root, **kwargs):
     Will not download TAR if it already exists.
     Will not extract the TAR if the data directory already exists in --deployment-root
     """
-    auth_with_gsutil(deployment_root)
-    download_indicoapi_data(deployment_root, version=version, extract=extract)
+    auth_with_gsutil()
+    download_indicoapi_data(version=version, extract=extract)
 
 
 for command_group in [single_node, gke, eks, aks]:
     infra.add_command(command_group)
+
+
+@infra.command("generate-metrics-cert")
+@click.pass_context
+def generate_metrics_cert(ctx):
+    """
+    Generate key/certificate for external metrics usage
+    """
+    crypto_output = generate_cert()
+    run_cmd("kubectl create ns custom-metrics > /dev/null 2>&1", silent=True)
+    if not update_secret("cm-adapter-serving-certs", namespace="custom-metrics"):
+        return
+    secho("Generating key/certificate", fg="green")
+    run_cmd(
+        f"""kubectl create secret generic cm-adapter-serving-certs -n custom-metrics --from-literal=serving.key="{crypto_output.get('key')}" --from-literal=serving.crt="{crypto_output.get('crt')}" """,
+        silent=True,
+    )
